import React from 'react';
export interface HullCfgStyle {
    /**
     * @description 填充颜色
     * @default 'lightblue'
     */
    fill: string;
    /**
     * @description 描边颜色
     * @default 'blue'
     */
    stroke: string;
    /**
     *
     * @description 透明度
     * @default 0.2
     */
    opacity: number;
}
export interface HullCfg {
    /**
     * @description 在包裹内部的节点实例或节点 Id 数组
     * @default []
     *
     */
    members: string[];
    /**
     * 包裹的 id
     */
    id?: string;
    /**
     * @description 包裹的类型
     * round-convex: 生成圆角凸包轮廓，
     * smooth-convex: 生成平滑凸包轮廓
     * bubble: 产生一种可以避开 nonMembers 的平滑凹包轮廓（算法）。
     * @default round-convex
     */
    type?: 'round-convex' | 'smooth-convex' | 'bubble';
    /** 不在轮廓内部的节点数组，只在 bubble 类型的包裹中生效 */
    nonMembers?: string[];
    /** 轮廓的样式属性 */
    style?: Partial<HullCfgStyle>;
    /**
     * @description 轮廓边缘和内部成员的间距
     * @default 10
     */
    padding?: number;
}
export interface IHullProps {
    /**
     * @description 配置
     */
    options: HullCfg[];
}
declare const Hull: React.FunctionComponent<IHullProps>;
export default Hull;
